/*******************************************************************************

 WINBINDER - A native Windows binding

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 Font functions

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "wb.h"

//-------------------------------------------------------------------- CONSTANTS

#define MAX_FONTS			256

//-------------------------------------------------------------------- VARIABLES

// Local

static PFONT pFonts[MAX_FONTS];				// Font cache
static int nInstalledFonts = 0;

//---------------------------------------------------------- FUNCTION PROTOTYPES

//----------------------------------------------------------- EXPORTED FUNCTIONS

/* Sets the font of control pwbobj. Returns the index of the font. */

int wbCreateFont(LPCTSTR pszName, int nHeight, COLORREF color, DWORD dwFlags)
{
	HFONT hFont;
	HDC hdc;

	if(nInstalledFonts >= MAX_FONTS - 1)
		return -1;

	// Convert points to pixels

    hdc = GetDC(NULL);
    nHeight = -MulDiv(nHeight, GetDeviceCaps(hdc, LOGPIXELSY), 72);
    ReleaseDC(NULL, hdc);

	// Create font with attributes

	hFont = CreateFont(nHeight,
	    0, 0, 0,
		dwFlags & FTA_BOLD ? FW_BOLD : FW_NORMAL,
		dwFlags & FTA_ITALIC,
		dwFlags & FTA_UNDERLINE,
		0, DEFAULT_CHARSET, 0,0,0,0,
		pszName);

	// Store font in cache

	if(hFont) {
		pFonts[nInstalledFonts] = wbMalloc(sizeof(FONT));
		if(!pFonts[nInstalledFonts])
			return -1;
		pFonts[nInstalledFonts]->pszName = wbMalloc(strlen(pszName) + 1);
		if(!pFonts[nInstalledFonts]->pszName)
			return -1;
		strcpy(pFonts[nInstalledFonts]->pszName, pszName);
		pFonts[nInstalledFonts]->nHeight = nHeight;
		pFonts[nInstalledFonts]->color = color;
		pFonts[nInstalledFonts]->dwFlags = dwFlags;
		pFonts[nInstalledFonts]->hFont = hFont;
		nInstalledFonts++;
	} else
		return -1;

	return nInstalledFonts - 1;
}

/* Sets the font of control pwbobj. pftFont points to a FONT structure
  with the font data to be set.

  TODO: change text color

  */

BOOL wbSetControlFont(PWBOBJ pwbo, int nFont, BOOL bRedraw)
{
	if(!wbIsWBObj(pwbo, TRUE))					// Is it a valid control?
		return FALSE;

	if(!IsWindow(pwbo->handle))
		return FALSE;

	if(nFont < 0 || nFont > nInstalledFonts - 1)
		return FALSE;

	SendMessage(pwbo->handle, WM_SETFONT, (WPARAM)pFonts[nFont]->hFont,
	  MAKELPARAM(bRedraw, 0));
/*
	// Doesn't work, should intercept WM_CTLCOLOR messages
{
	HDC hdc;
    hdc =  GetWindowDC(pwbo->handle);
	SetTextColor(hdc, pFonts[nFont]->color);
	printf("%d %d %08X\n", hdc, pwbo->handle, pFonts[nFont]->color);
    ReleaseDC(pwbo->handle, hdc);
}
*/

	return TRUE;
}

// TODO: Not tested, not exported to PHP

BOOL wbDestroyFonts(void)
{
	int i;

	for(i = 0; i < nInstalledFonts; i++) {
		if(!DeleteObject(pFonts[i]->hFont))
			return FALSE;
		if(!wbFree(pFonts[i]->pszName))
			return FALSE;
		if(!wbFree(pFonts[i]))
			return FALSE;
	}
	nInstalledFonts = 0;
	return TRUE;
}

//------------------------------------------------------------------ END OF FILE
